/*********************************************************************************/
/* Unterprogram zum ansprechen eines GrafikLCD mit KS0713             06.01.2006 */
/*                                                                               */
/* Es wird das Display 64128E von Displaytech verwendet. 128x64Pixel             */
/* Achtung das Datenblatt von Displaytech ist sehr schlecht und das LCD          */
/* ist sehr empfindlich gegen falsche Programmierung.                            */
/*                                                                               */
/* Das Display wird seriell angesprochen. Leider laesst es sich dann nicht       */
/* auslesen und braucht ein dickes Schattenregister von 1024Bytes                */
/* Deshalb wird hier mit einem Trick gearbeitet, dem genialen Olaf-Algorythmus.  */
/* Dabei merkt man sich immer die zuletzt genutzten Bytes in einem kleineren     */
/* Puffer und hofft ansonsten das beste. :-)                                     */
/*                                                                               */
/*  G-Bus  KS0713   M8C/13Port                                                   */
/*    1      GND      GND                                                        */
/*    2      5V       5V                                                         */
/*    3      SID      TxD0/P14                                                   */
/*    4      /CS1B    P13                                                        */
/*    5      RS       P17                                                        */
/*    6      SCLK     CLK/P16                                                    */
/*                                                                               */
/* Es wird entweder der syncrone UART0 des R8/13 benutzt, oder von Hand an       */
/* beliebigen Ports rumgefummelt                                                 */
/*                                                                               */
/* Wird der Uart benutzt so laeuft der Takt mit 1.6Mhz. Das muss die Verdrahtung */
/* mitmachen, ausserdem sollte die CLK Leitung terminiert sein. Dafuer hat sich  */
/* 100 Ohm + 100pF nach Masse als brauchbar erwiesen                             */
/*                                                                               */
/* Da das Display mit 3.3V laeuft wird noch ein NJU7223DL1-33 benoetigt. Man     */
/* kann ihn z.B bei Akizukidenshi in Akihabara kaufen. Der Pegel der Daten-      */
/* leitungen wird mit 330+680 Ohm angepasst. An /Reset haengt 47k+220nF          */
/* Die RD und WR Leitungen muessen auf GND gelegt werden.                        */
/* Ausserdem waere es natuerlich moeglich den R8C auch mit 3.3V laufen zu lassen */
/* und sich die Pegelwandlung zu sparen. Sollte man zufaellig nicht nach Tokyo   */
/* einkaufen kommen kann man vielleicht auch einen LM317 und zwei Widerstaende   */
/* nehmen.                                                                       */
/*                                                                               */
/*********************************************************************************/

/* Wenn dieser Eintrag definiert ist wird der Uart0 benutzt */
#define SYNCUART0

#include <stdio.h>
#include <stdlib.h>   
#include <stdarg.h>
#include <string.h>

#include "sfr_r813.h"    // special funktion register


#define KS0713
#include "ks0713.h"
#undef  KS0713

/* Benutzte Portleitungen */
void set_SI(void)   { p1_4 = 1; }
#define SET_SI p1_4 = 1
void reset_SI(void) { p1_4 = 0; }
#define RESET_SI p1_4 = 0
void set_CS(void)   { p1_3 = 1; }
void reset_CS(void) { p1_3 = 0; }
void set_A0(void)   { p1_7 = 1; }
void reset_A0(void) { p1_7 = 0; }
void set_SCL(void)  { p1_6 = 1; }
#define SET_SCL p1_6 = 1
void reset_SCL(void){ p1_6 = 0; }
#define RESET_SCL p1_6 = 0

void mdelay(unsigned short timer) 
{ unsigned short t; 
    
   t=timer; 
   while(t--); 
 
}

/* Port Initialisierung */
#ifdef SYNCUART0
/* Benutzt den syncronen Uart0 eines R8C13 */
void ks0713_port(void)
{
  pd1 |= 0xd8;    /* Alle genutzten Port auf Ausgang schalten */
  drr = 0xff;     /* Treiber auf volle Suppe */

  set_SCL();

  u0c0  = 0x88;   /* MSB first  UFORM =1                        */
                  /* Bit mit fallender Flanke ausgeben CKPOL=0  */
                  /* CMOS Ausgang NCH=0                         */
                  /* f1SIO als Clock  CLK0=0, CLK1=0            */

  u0brg = 0x05;   /* 0x05 Bitrate von 1.6MHz */

  u0mr  = 0x01;   /* Parity ausgeschaltet PRYE=0                */
                  /* Interner Clock CKDIR=0                     */
                  /* Clocksync-Mode SMD2=0, SMD1=0, SMD0=1      */


  u0c1  = 0x03;   /* Ausgang einschalten, Eingang ausschalten   */


  set_CS();
  reset_A0();
   
}
#else
/* Mit jedem beliebigen Portpin aber langsamer */
void ks0713_port(void)
{
  pd1 |= 0xd8; /* Alle genutzten Port auf Ausgang schalten */
  drr = 0xff;
  set_CS();
  reset_SI();
  set_SCL();
  reset_A0();
}
#endif


/* Ein Byte seriell ans Display ausgeben */
#ifdef SYNCUART0
void ks0713_write(unsigned char data)
{  unsigned char i;

 
   reset_CS();

   u0tb =  data;  
 /*  u0c1 &= ~0x02;  */

   while ( (u0c1 & 0x02) == 0x00);   /* Warten bis fertig */
   while ( (u0c0 & 0x08) == 0x00); 
   /* Es waer ja eigentlich intelligenter nicht zu warten, sondern       */
   /* vor dem rausschreiben an das SPI zu schauen ob es schon fertig ist */
   /* Leider moegen es aber die Displaycontroller nicht wenn man denen   */
   /* CS zu frueh abstellt.                                              */
   set_CS();
 
}
#else
void ks0713_write(unsigned char data)
{  unsigned char i;

 
   reset_CS();

  for (i=8;i>0;i--) 
   { /* MSB ausgeben */
    if (data & 0x80) SET_SI;
    else             RESET_SI;

    RESET_SCL; 
    data =  data  << 1  ;  /* Ein Bit nach links schieben */

    /* Zum naechsten Bit takten */
    SET_SCL;

   } 

   set_CS();
 
}
#endif

/***************************************************************************/



/* Schreibt ein Byte ins Controlregister */
void ks0713_writecontrol(unsigned char data)
{ reset_A0();
  ks0713_write(data);
}

/* Schreibt ein Byte ins Display */
void ks0713_writedata(unsigned char data)
{ set_A0();
  ks0713_write(data);
}

/* Page auswahl des Display von 0 bir 7               */
/* Dabei ist 0 der obere Zeilenblock und 7 ganz unten */
/* Eigentlich ist das Display ganz gut organisiert    */
void ks0713_pageadr(unsigned char data)
{
  ks0713_writecontrol((data & 0x0f) | 0xb0);
}

/* Spaltenauswahl, 0 ist ganz links und 127 ganz rechts */
void ks0713_columnadr(unsigned char data)
{  
  ks0713_writecontrol(((data & 0xf0)>>4) | 0x10); /* Highnibble */
  ks0713_writecontrol(data & 0x0f);               /* LowNibble  */
}

/* Bufferposition freigeben */
void clear_lcdram(void)
{ int i;

  for(i=0;i<LCDSIZE;i++) {
		lcdram[i].adr  =0xffff;
	    lcdram[i].data =0x00;
      }

}

/* Display  loeschen                             */
void ks0713_clrscr(void)
{ unsigned char i,j;

  for (i=0;i<8;i++)
  { 
    ks0713_pageadr(i);
    for (j=0;j<128;j++)
    {
       ks0713_columnadr(j);
       ks0713_writedata(0x00); 
    } 
  }
 clear_lcdram();
}


/* Diese Funktion merkt sich ob ein uebergebens Pixel im Pagebyte gesetzt ist */
/* und liefert das neu zu setzende Pagebyte zurueck.                          */
/* Leider ist aber fuer das gesammte Display 1kByte Ram notwendig. Da wir     */
/* nicht soviel haben gibt es einen mitlaufenden Buffer indem immer die       */
/* letzten Bytes gemerkt werden.                                              */
unsigned char bitbuffer(unsigned char x, unsigned char y, unsigned char data, 
                        unsigned char color)
{ int bufpos=0;
  unsigned short adr;

  /* Damit sieht man mal wofuer Zeiger gut sind und warum Java hirntod ist        */
  /* Es werden zwei char zu einem short zusammengefasst weil der R8/13 ein 16Bit  */
  /* Prozessor ist */
  *((unsigned char*) &adr)   = y;
  *((unsigned char*) &adr+1) = x;


  /* Buffer durchsuchen */
  while ( (lcdram[bufpos].adr!=adr) && (bufpos<LCDSIZE)) bufpos++;

  /* Bingo! Das Byte ist im Buffer */
  if (bufpos<LCDSIZE) {
     if (color) lcdram[bufpos].data |= data;
	       else lcdram[bufpos].data &= ~data;

     return(lcdram[bufpos].data); 
  }

  /* Mist! Neues Byte, also Buffer shiften */
  for(bufpos=LCDSIZE-1;bufpos>0;bufpos--) lcdram[bufpos] = lcdram[bufpos-1];
    
  /* Das neuste Byte bekommt immer die erste Bufferposition damit es */
  /* beim suchen auch schnell gefunden wird */ 
  lcdram[0].adr  = adr;

  if (color==0) lcdram[0].data = 0x00; 
  else          lcdram[0].data = data;

  return(lcdram[0].data);

}

/* Pixel auf die gewuenschte Farbe setzen.   */
/* color: 0 = Weiss, 1  = Schwarz            */
/* x=0, y=0 linke obere bildschirmecke       */
/* x=127, y=63 rechte untere Bildschirmecke   */
void ks0713_setpixel(unsigned char x, unsigned char y, unsigned char color)
{ unsigned char bit;
  unsigned char data;
  /* Bereichspruefung, sicherer kostet aber Zeit */
  /* if ((x>127) || (y>63)) return; */

  ks0713_columnadr(x); /* X-Adresse ist einfach */
  
  bit = y>>3;
  ks0713_pageadr(bit); /* Es gibt 8 Pages */
 
  switch(y - (bit<<3)) 
  {
    case 0x00 : if (color) data= bitbuffer(x,bit,0x01,color);
		        else       data= bitbuffer(x,bit,0x01,0);
		 break;
    case 0x01 : if (color) data= bitbuffer(x,bit,0x02,color);
		        else       data= bitbuffer(x,bit,0x02,0);
		 break;
    case 0x02 : if (color) data= bitbuffer(x,bit,0x04,color);
		        else       data= bitbuffer(x,bit,0x04,0);
		 break;
    case 0x03 : if (color) data= bitbuffer(x,bit,0x08,color);
		        else       data= bitbuffer(x,bit,0x08,0);
		 break;
    case 0x04 : if (color) data= bitbuffer(x,bit,0x10,color);
		        else       data= bitbuffer(x,bit,0x10,0);
		 break;
    case 0x05 : if (color) data= bitbuffer(x,bit,0x20,color);
		        else       data= bitbuffer(x,bit,0x20,0);
		 break;
    case 0x06 : if (color) data= bitbuffer(x,bit,0x40,color);
		        else       data= bitbuffer(x,bit,0x40,0);
		 break;
    case 0x07 : if (color) data= bitbuffer(x,bit,0x80,color);
		        else       data= bitbuffer(x,bit,0x80,0);
		 break;

 
    
  }

  ks0713_writedata(data);
  
}

/* Schwarz auf weiss */
void ks0713_normal(void)
{
   ks0713_writecontrol(0xa6); /* Display nicht invertiert */
}
  
/* Display invertieren */
void ks0713_invert(void)
{
   ks0713_writecontrol(0xa7); /* Display invertiert */
}
  


/* Initialisiert das Display */
/* Die Reihenfolge ist sehr wichtig sonst killt der Kontroller        */
/* das LCD ohne das man etwas merkt. Sollte man nichts auf dem LCD    */
/* sehen dann sofort abschalten und dann erst ueberlegen woran es     */
/* liegt!                                                             */
void ks0713_init(void)
{
   ks0713_port();

  
   ks0713_writecontrol(0xaf); /* Display ein              */
   ks0713_writecontrol(0x40); /* Start Display Line = 0   */
   ks0713_writecontrol(0xa0); /* ADC = invers             */
   ks0713_writecontrol(0xa3); /* LCD-Bias = 1/9           */
   ks0713_writecontrol(0xc0);  /* Outputstatus = 0        */

   ks0713_writecontrol(0x2c); /* LCD Spannung ein         */
   mdelay(2000);

   ks0713_writecontrol(0x2e); /* LCD Spannung ein         */
   mdelay(2000);

   ks0713_writecontrol(0x2f); /* LCD Spannung ein         */
   mdelay(2000);

   ks0713_writecontrol(0x25);
   ks0713_writecontrol(0xa4); /* Displaytest aus          */
   ks0713_writecontrol(0xa6); /* Display nicht invertiert */

   ks0713_writecontrol(0x81); /* Reference Voltage Mode   */
   ks0713_writecontrol(0x3f); /* Reference Voltage        */



   /* Bildschirm loeschen */
   ks0713_clrscr();
 

}



