/****************************************************************************/
/*                                                                  6.01.06 */
/* Grafikroutinen zur Ausgabe von Text und Pixel/Lines/Circle auf ein       */
/* Grafik-LCD.                                                              */
/*                                                                          */
/****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define font8x8 
#include "lcdg.h"
#include "char8.h"
#include "ks0713.h"


/*===========================================================================*/

unsigned char lcdg_xpos; /* Merker fuer die Postion von Text auf dem Grafikscreen*/
unsigned char lcdg_ypos;

void lcdg_init()
{
	ks0713_init();		/* Initialisiert das Grafikdisplay       */
}

/*****************************************************************************/
/* Zeichnet ein Pixel in der gewuenschten Farbe                              */
/*****************************************************************************/
void lcdg_drawpixel(unsigned char x, unsigned char y, unsigned char color)
{  

   /* Diese Funktion ist Hardwarespezifisch*/
   /* Uebergabe an den Displayprozessor */

   ks0713_setpixel( x, y, color);

}



/*---------------------------------------------------------------------------*/
/* Ab hier nur hardwareunabhaengige Funktionen                               */
/*---------------------------------------------------------------------------*/

/*****************************************************************************/
/* Interne Hilfsroutine zum kreiszeichen, spiegelt die Segmente              */
/*****************************************************************************/

void circlepoints(unsigned char cx, unsigned char cy, unsigned char x,
                  unsigned char y, unsigned char color)
{
   if (x == 0) {
      lcdg_drawpixel(cx,cy+y, color);
      lcdg_drawpixel(cx,cy-y, color);
      lcdg_drawpixel(cx+y,cy, color);
      lcdg_drawpixel(cx-y,cy, color);
   } else
   if (x == y) {
      lcdg_drawpixel(cx+x,cy+y, color);
      lcdg_drawpixel(cx-x,cy+y, color);
      lcdg_drawpixel(cx+x,cy-y, color);
      lcdg_drawpixel(cx-x,cy-y, color);
   } else
   if (x<y) {
      lcdg_drawpixel(cx+x,cy+y, color);
      lcdg_drawpixel(cx-x,cy+y, color);
      lcdg_drawpixel(cx+x,cy-y, color);
      lcdg_drawpixel(cx-x,cy-y, color);
      lcdg_drawpixel(cx+y,cy+x, color);
      lcdg_drawpixel(cx-y,cy+x, color);
      lcdg_drawpixel(cx+y,cy-x, color);
      lcdg_drawpixel(cx-y,cy-x, color);
   }
}


/*****************************************************************************/
/* Zeichnet einen Kreis um die angebene Position                             */
/*****************************************************************************/

void lcdg_circle(unsigned char xCenter, unsigned char yCenter, 
                 unsigned char radius, unsigned char color)
{
   unsigned char x= 0;
   unsigned char y= radius;
   signed char p= (5- radius*4)/4;

   circlepoints(xCenter, yCenter, x, y, color);
   while (x<y) {
      x++;
      if (p<0) {
	 p += 2 * x + 1;
      } else {
	 y--;
	 p += 2 * (x-y)+1;
      }
      circlepoints(xCenter, yCenter, x, y, color);
   }
}


/*****************************************************************************/
/*  Linie von (x1, y1) nach (x2, y2) zeichnen.                               */
/*  Erwartet das  x1 < x2 und die Steigung kleiner 1.                        */
/*****************************************************************************/
void lcdg_line(int Ax, int Ay, int Bx,
               int By, unsigned char Color)
{

	 int dX;	
	 int dY;

	int Xincr, Yincr;

    dX = abs(Bx-Ax);
    dY = abs(By-Ay);

	if (Ax > Bx) { Xincr=-1; } else { Xincr=1; }
	if (Ay > By) { Yincr=-1; } else { Yincr=1; }

	if (dX >= dY)	
	{
		int dPr 	= dY<<1;         
		int dPru 	= dPr - (dX<<1); 
		int P 		= dPr - dX;  

		for (; dX>=0; dX--)          
		{
			lcdg_drawpixel(Ax, Ay, Color); 
			if (P > 0)             
			{
				Ax+=Xincr;
				Ay+=Yincr;
				P+=dPru;  
			}
			else              
			{
				Ax+=Xincr;
				P+=dPr;   
			}
		}
	}
	else            
	{
		int dPr 	= dX<<1;     
		int dPru 	= dPr - (dY<<1); 
		int P 		= dPr - dY; 

		for (; dY>=0; dY--)         
		{
			lcdg_drawpixel(Ax, Ay, Color);
			if (P > 0)            
			{
				Ax+=Xincr;    
				Ay+=Yincr;    
				P+=dPru;      
			}
			else                  
			{
				Ay+=Yincr;    
				P+=dPr;       
			}
		}
	}
}



/*************************************************************************/
/* Zeichnet eine Kiste                                                   */
/*************************************************************************/

void lcdg_box(int x1, int y1, int x2, int y2, int color )
{
     lcdg_line(x1,y1,x2,y1,color);
     lcdg_line(x1,y2,x2,y2,color);
     lcdg_line(x1,y1,x1,y2,color);
     lcdg_line(x2,y1,x2,y2,color);
}


/*************************************************************************/
/* Zeichnet eine gefuellte Kiste                                         */
/*************************************************************************/

void lcdg_fillbox(int x1, int y1, int x2, int y2, int color )
{ int i;
 for(i=x1;i<=x2;i++) {lcdg_line(i,y1,i,y2,color);};
}


/*****************************************************************************/
/* Setzt den Charaktercursor an eine bestimmte Pixelposition                 */
/* Dabei ist 0,0 die linke obere Ecke			   		                     */
/*****************************************************************************/
void lcdg_gotoxy(int x, int y)
{ 
   lcdg_xpos = 8 * x;
   lcdg_ypos = 8 * y + 8;
}


void lcdg_gotoxypixel(int x, int y)
{
   lcdg_xpos = x;
   lcdg_ypos = y;
}

/*****************************************************************************/
/* Liefert maximale Zeichenanzahl in X-Richtung                              */
/*****************************************************************************/
int lcdg_maxx(void)
{
  return(MAXLCDG_X / 8);   
}


/*****************************************************************************/
/* Liefert maximale Zeichenanzahl in Y-Richtung                              */
/*****************************************************************************/
int lcdg_maxy(void)
{
  return(MAXLCDG_Y / 8);   
}


/*****************************************************************************/
/* Setzt den Cursor ein Zeichen weiter. Dabei wird am Zeilenende automatisch */
/* ein Zeilenumbruch durchgefuehrt                                           */
/*****************************************************************************/
void lcdg_incpos(void)
{
 
   lcdg_xpos+=8;
   if (lcdg_xpos>MAXLCDG_X) {
      lcdg_xpos=0;
      lcdg_ypos+=8;
   }
   if (lcdg_ypos>MAXLCDG_Y) lcdg_ypos=0;
   
}



/*************************************************************************/
/* Fuert ein CRLF fuer den Grafikcursor aus                              */
/*************************************************************************/
void lcdg_returnpos(void)
{
 lcdg_xpos=0;
 lcdg_ypos+=8;
 if (lcdg_ypos>MAXLCDG_Y) lcdg_ypos=8;
}  

/*****************************************************************************/
/* Gibt ein Zeichen als Grafik aus                                           */
/*****************************************************************************/
void lcdg_putc(unsigned char asci,int x, int y, int color)
{
 int i,zeiger,colfg,colbg;
 unsigned char bitblock;
 zeiger = asci * 8;
 colfg=color;
 if (colfg==LCDG_BLACK) colbg=LCDG_WHITE; else colbg=LCDG_BLACK;

 for (i  = 0; i<8; i++) {
    bitblock = char8x8[zeiger+i];
    if ((bitblock & 0x80) == 0x80) lcdg_drawpixel(x,y-7+i,   colfg);
			      else lcdg_drawpixel(x,y-7+i,   colbg);
    if ((bitblock & 0x40) == 0x40) lcdg_drawpixel(x+1,y-7+i, colfg);
			      else lcdg_drawpixel(x+1,y-7+i, colbg);
    if ((bitblock & 0x20) == 0x20) lcdg_drawpixel(x+2,y-7+i, colfg);
			      else lcdg_drawpixel(x+2,y-7+i, colbg);
    if ((bitblock & 0x10) == 0x10) lcdg_drawpixel(x+3,y-7+i, colfg);
			      else lcdg_drawpixel(x+3,y-7+i, colbg);
    if ((bitblock & 0x08) == 0x08) lcdg_drawpixel(x+4,y-7+i, colfg);
			      else lcdg_drawpixel(x+4,y-7+i, colbg);
    if ((bitblock & 0x04) == 0x04) lcdg_drawpixel(x+5,y-7+i, colfg);
			      else lcdg_drawpixel(x+5,y-7+i, colbg);
    if ((bitblock & 0x02) == 0x02) lcdg_drawpixel(x+6,y-7+i, colfg);
			      else lcdg_drawpixel(x+6,y-7+i, colbg);
    if ((bitblock & 0x01) == 0x01) lcdg_drawpixel(x+7,y-7+i, colfg);
			      else lcdg_drawpixel(x+7,y-7+i, colbg);
 }
}



/* Ausgabe eines Zeichens und Cursor eine Stelle weiter */
void write_LCDchar(unsigned char data)
{
   lcdg_putc(data,lcdg_xpos,lcdg_ypos, BLACK);
   lcdg_incpos();

}

/* Zahl beliebiger Basis auf den Bildschirm schreiben */
void itoaprint(int x, char basis)
{ char buffer[6];
  int  b=5;

  if (x<0) { write_LCDchar('-'); x = -x; }

  for (; x; b--, x /= basis)
      if ((buffer[b] = x%basis + '0') > '9') buffer[b] += 7;       

  do { write_LCDchar(buffer[++b]); } while (b<5);

}

/*******************************************************************/
/* Vereinfachte printf Funtkion. Kann zwar viel weniger als das    */
/* Original, braucht dafuer aber auch viel weniger RAM             */
/*                                                                 */
/* Erkennt nur String, und Ints als Variablen, keine float!        */
/* Sollte gelegentlich nochmal verbessert werden um auch fuehrende */
/* Nullen ausgeben zu koennen. Und feste Ausgabeformate waeren     */
/* ganz nett.                                                      */
/*                                                                 */
/* Der Parameter q gibt die Pixelzeile des Displays an. Dabei ist 0*/
/* die oberste Zeile im Display. Es sollte aber noch vom User die  */
/* Hoehe eines Zeichens dazu addiert werden. q=8 ->erste Zeile     */
/*                                                                 */
/* Der Parameter z gibt die Spalte des Displays an. Dabei ist 0 die*/
/* Pixelspalte am linken Rand.                                     */ 
/*                                                                 */
/* Funktion arbeitet recursiv. Der Userstack sollte dafuer gross   */
/* genug sein oder es knallt.                                      */
/*******************************************************************/
void oprintf(char *fmt, ...)
{
 va_list ap;
 char *p, *sval;
 int ival;

 va_start(ap,fmt); 
 for (p =fmt; *p; p++) {
   if (*p != '%') { write_LCDchar(*p);  continue; }
   switch (*++p) {
   case 'd' :
   case 'i' :
        ival = va_arg(ap,int);
        itoaprint(ival,10); 
        break;
   case 'x' :
        ival = va_arg(ap,int);
        itoaprint(ival,16); 
        break;
   case 's' :
   	    for (sval=va_arg(ap, char *); *sval; sval++)
			write_LCDchar(*sval); 
        break;
    case 'q' :
        ival = va_arg(ap,int);
  	    lcdg_gotoxypixel(lcdg_xpos,ival);
        break;
    case 'z' :
        ival = va_arg(ap,int);
  	    lcdg_gotoxypixel(ival, lcdg_ypos);
        break;
 
   default:
        write_LCDchar(*p);
        break; 	
   }
 }
 va_end(ap);
}




